#encoding:UTF8
from BaseDataMaintenance.dataSource.pool import ConnectorPool
from BaseDataMaintenance.dataSource.source import *
from BaseDataMaintenance.common.Utils import *
import queue
from tablestore import *
from multiprocessing import RLock
from threading import Thread
from apscheduler.schedulers.blocking import BlockingScheduler
from BaseDataMaintenance.common.multiThread import MultiThreadHandler
from BaseDataMaintenance.model.ots.proposedBuilding_tmp import proposedBuilding_tmp
from BaseDataMaintenance.model.ots.designed_project import designed_project
import traceback

class DataSynchronization():

    def __init__(self):
        self.done_lock = RLock()
        self.isDone = False
        self.proposedBuilding_table = "proposedBuilding_tmp"
        self.proposedBuilding_table_index = "proposedBuilding_tmp_index"
        self.pool_ots = ConnectorPool(init_num=10,max_num=40,method_init=getConnect_ots)

    def producer(self,task_queue):
        '''
        :return:生产数据
        '''
        ots_client = getConnect_ots()

        bool_query = BoolQuery(must_queries=[ExistsQuery("crtime")])

        columns = ["uuid","crtime","json_list_group"]
        rows, next_token, total_count, is_all_succeed = ots_client.search(self.proposedBuilding_table, self.proposedBuilding_table_index,
                                                                          SearchQuery(bool_query ,sort=Sort(sorters=[FieldSort("crtime",SortOrder.ASC)]), limit=100, get_total_count=True),
                                                                          ColumnsToGet(columns,return_type=ColumnReturnType.SPECIFIED))
        list_data = getRow_ots(rows)
        for _data in list_data:
            _proposed = proposedBuilding_tmp(_data)
            task_queue.put(_proposed,True)
        while next_token:
            rows, next_token, total_count, is_all_succeed = ots_client.search(self.proposedBuilding_table, self.proposedBuilding_table_index,
                                                                              SearchQuery(bool_query ,next_token=next_token, limit=100, get_total_count=True),
                                                                              ColumnsToGet(columns,return_type=ColumnReturnType.SPECIFIED))
            list_data = getRow_ots(rows)
            for _data in list_data:
                _proposed = proposedBuilding_tmp(_data)
                task_queue.put(_proposed,True)

    def comsumer(self,task_queue):

        def _handle(_proposed,result_queue,pool_ots):
            ots_client = pool_ots.getConnector()

            #修改designed_project
            _project_dict = _proposed.toDesigned_project(ots_client)

            try:
                if _project_dict is not None:
                    #更新数据
                    _designed_project = designed_project(_project_dict)
                    _designed_project.update_project(ots_client)

                #删除tmp
                _proposed.delete_row(ots_client)
            except Exception as e:
                log("comsumer failed cause of %s"%(str(e)))
                log(traceback.format_exc())



            pool_ots.putConnector(ots_client)


        result_queue = queue.Queue()

        mt = MultiThreadHandler(task_queue,_handle,result_queue,thread_count=30,pool_ots=self.pool_ots)
        mt.run()


    def waitTask(self,task_queue):
        for i in range(60):
            if task_queue.qsize()>0:
                return True
            else:
                time.sleep(1)
        return False

    def maxcompute2ots(self):

        task_queue = queue.Queue()

        self.producer(task_queue)

        if self.waitTask(task_queue):
            thread_comsumer = Thread(target=self.comsumer,args=([task_queue]))
            thread_comsumer.start()


    def scheduler(self):
        _scheduler = BlockingScheduler()
        _scheduler.add_job(self.maxcompute2ots,"cron",minute="*/10")
        _scheduler.start()

def startSychro():
    ds = DataSynchronization()
    ds.scheduler()

if __name__=="__main__":
    ds = DataSynchronization()
    ds.scheduler()


